﻿using EventTracker.Mapping.EventTrackerService;
using EventTracker.Mapping.ViewModels;
using System;
using System.Data.Services.Client;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using Telerik.Windows.Controls.Map;

namespace EventTracker.Mapping
{
    public partial class MainPage : UserControl
    {
        private DataServiceQuery<Conference> query;
        private BingGeocodeProvider geocodeProvider;
        private string baseUrl = "";
        private InfoBalloon balloon;

        public MainPage()
        {
            InitializeComponent();

            if (App.Current.Resources["appPath"] != null)
                baseUrl =  App.Current.Resources["appPath"].ToString();

            var data = new EventTrackerService.DataManager(new Uri(baseUrl + "Services/EventTrackerService.svc/", UriKind.Relative));
            query = data.CreateQuery<Conference>("Conferences");
            query.BeginExecute(new AsyncCallback(OnConferencesLoaded), null);

            geocodeProvider = new BingGeocodeProvider();
            geocodeProvider.ApplicationId = ((BingMapProvider)this.radMap1.Provider).ApplicationId;
            geocodeProvider.MapControl = this.radMap1;
            geocodeProvider.GeocodeCompleted += new EventHandler<GeocodeCompletedEventArgs>(geocodeProvider_GeocodeCompleted);
        }

        void geocodeProvider_GeocodeCompleted(object sender, GeocodeCompletedEventArgs e)
        {
            GeocodeResult result = e.Response.Results.First();

            var conference = (Conference)e.Response.RequestUserData;

            var poi = CreatePoint(result, conference);

            this.informationLayer.Items.Add(poi);
        }

        private PointOfInterest CreatePoint(GeocodeResult result, Conference conference)
        {
            PointOfInterest poi = new PointOfInterest();
            poi.Location = new Location(result.Locations.First().Latitude, result.Locations.First().Longitude);
            poi.Title = conference.Name;
            poi.Description = conference.Description;
            poi.Link = new Uri(baseUrl + "Conferences/Details/" + conference.ID, UriKind.Relative);
            return poi;
        }

        private void OnConferencesLoaded(IAsyncResult ar)
        {
            var conferences = query.EndExecute(ar);

            foreach (Conference c in conferences)
            {
                geocodeProvider.GeocodeAsync(new GeocodeRequest() { Query = c.Location, UserData = c });
            }
        }

        private PointOfInterest currentPOI;

        private void PinClicked(object sender, MouseButtonEventArgs e)
        {
            var mapShape = sender as FrameworkElement;

            if (mapShape != null)
            {
                var poi = (PointOfInterest)mapShape.Tag;

                EnsureBalloon();
                SetUpBalloon(poi);
            }
        }
  
        private void EnsureBalloon()
        {
            if (balloon == null)
            {
                balloon = new InfoBalloon();
                this.informationLayer.Items.Add(balloon);
            }
        }
  
        private void SetUpBalloon(PointOfInterest poi)
        {
            if (poi == currentPOI && this.balloon.Visibility== System.Windows.Visibility.Visible)
            {
                this.balloon.Visibility = System.Windows.Visibility.Collapsed;
            }
            else
            {
                this.balloon.Visibility = System.Windows.Visibility.Visible;
                balloon.DataContext = poi;
                MapLayer.SetLocation(balloon, poi.Location);
                currentPOI = poi;
            }
        }
    }
}