﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq;
using System.Web.Mvc;
using EventTracker.Infrastructure;
using EventTracker.Models;
using Telerik.OpenAccess.Diagnostics;

namespace EventTracker.Controllers
{
    public class ToolsController : Controller
    {
        private readonly IContextManager contextManager;

        public ToolsController(IContextManager contextManager)
        {
            this.contextManager = contextManager;
        }
        public ActionResult Index()
        {
            return View();
        }

        public ActionResult MetricWatcher()
        {
            return View();
        }

        public ActionResult EventWatcher()
        {

            var history = new History(contextManager.Context);

            var list = history.GetDatabaseEvents(null, null, null);

            if (list != null && list.Count > 0)
            {
                List<ITraceEvent> filteredList = new List<Telerik.OpenAccess.Diagnostics.ITraceEvent>(list.Count);
                foreach (var entry in list)
                {
                    filteredList.Add(entry);
                }

                return View(list);
            }

            return View();
        }
        public JsonResult MetricWatcherData()
        {
            var metricEntries = GetMetricHistory();

            if (metricEntries == null || metricEntries.Count < 2)
            {
                return Json(new List<MetricViewModel>());
            }

            //int entitiesToTake = 6;
            //var metricList = metricEntries.OrderByDescending(m => m.Timestamp)
            //                              .ToList();

            //if (metricEntries.Count > entitiesToTake)
            //{
            //    metricList.RemoveRange(entitiesToTake, metricEntries.Count - entitiesToTake);
            //}

            //

            var transfo = new DatabaseMetricsTransformation<MetricViewModel>(metricEntries);
            transfo.SetTimeStamp("TimeStamp");
            transfo.Calculate("Connection_Obtained", "MetricDelta", "Connection_Obtained");
            transfo.Calculate("Sql_Query", "MetricDelta", "Sql_Query");
            transfo.Calculate("Sql_Commit", "MetricDelta", "Sql_Commit");
            transfo.Calculate("Sql_FetchedRows", "MetricDelta", "Sql_FetchedRows");
            transfo.Calculate("Cache_Object_Hits", "MetricDelta", "Cache_Object_Hits");
            transfo.Calculate("Cache_Object_Miss", "MetricDelta", "Cache_Object_Miss");
            transfo.Calculate("Cache_Query_Hits", "MetricDelta", "Cache_Query_Hits");
            transfo.Calculate("Cache_Query_Miss", "MetricDelta", "Cache_Query_Miss");
            transfo.Calculate("Sql_Delete", "MetricDelta", "Sql_Delete");
            transfo.Calculate("Sql_Insert", "MetricDelta", "Sql_Insert");
            transfo.Calculate("Sql_Update", "MetricDelta", "Sql_Update");
            transfo.Calculate("L2Cache_Query_HitRatio", "MetricDelta", "L2Cache_Query_HitRatio");
            transfo.Calculate("L2Cache_Query_FillRatio", "MetricDelta", "L2Cache_Query_FillRatio");
            transfo.Calculate("L2Cache_Object_HitRatio", "MetricDelta", "L2Cache_Object_HitRatio");
            transfo.Calculate("L2Cache_Object_FillRatio", "MetricDelta", "L2Cache_Object_FillRatio");
            transfo.Calculate("L2Cache_Query_Hit", "MetricDelta", "L2Cache_Query_Hit");
            transfo.Calculate("L2Cache_Query_Miss", "MetricDelta", "L2Cache_Query_Miss");
            transfo.Calculate("L2Cache_Object_Hit", "MetricDelta", "L2Cache_Object_Hit");
            transfo.Calculate("L2Cache_Object_Miss", "MetricDelta", "L2Cache_Object_Miss");
            transfo.Calculate("Connection_ObtainedWait", "MetricDelta", "Connection_ObtainedWait");
            transfo.Calculate("Connection_Active", "MetricDelta", "Connection_Active");
            transfo.Calculate("Connection_MaxActive", "MetricDelta", "Connection_MaxActive");
            transfo.Calculate("Connection_Idle", "MetricDelta", "Connection_Idle");
            transfo.Calculate("Connection_Validate", "MetricDelta", "Connection_Validate");
            transfo.Calculate("Context_Active", "MetricDelta", "Context_Active");
            transfo.Calculate("Context_MaxActive", "MetricDelta", "Context_MaxActive");
            transfo.Calculate("Context_Idle", "MetricDelta", "Context_Idle");

            //var items = transfo.OrderBy(m => m.TimeStamp)
            //                   .Select(m => new MetricViewModel(m));

            return Json(transfo.OrderByDescending(m=>m.TimeStamp).Take(6).OrderBy(m=>m.TimeStamp));
        }


        public ActionResult DatabaseManager()
        {
            return View();
        }

        private DatabaseMetricsCollection GetMetricHistory()
        {
            return new History(this.contextManager.Context).GetDatabaseMetrics(DateTime.Now.AddSeconds(-3), null, null);
        }

    }
}
