﻿using System.Web.Mvc;
using System.Web.Routing;
using System.Web.Security;
using EventTracker.Infrastructure;
using StructureMap;

namespace EventTracker
{
    // Note: For instructions on enabling IIS6 or IIS7 classic mode, 
    // visit http://go.microsoft.com/?LinkId=9394801

    public class MvcApplication : System.Web.HttpApplication
    {
        /// <summary>
        /// Ensures the database.
        /// </summary>
        private void EnsureDatabase()
        {
            var contextManager = ObjectFactory.GetInstance<ContextManager>();

            if (!contextManager.Context.Scope.Transaction.IsActive)
            {
                var schemaHandler = contextManager.Context.GetSchemaHandler();
                if (!schemaHandler.DatabaseExists() && Request.Path != "/Utilities/DatabaseManager")
                {
                    Response.Redirect("~/Utilities/DatabaseManager");
                }
            }
        }

        public static void RegisterRoutes(RouteCollection routes)
        {
            routes.IgnoreRoute("{*allsvc}", new { allsvc = @".*\.svc(/.*)?" });

            routes.IgnoreRoute("{resource}.axd/{*pathInfo}");

            routes.IgnoreRoute("Utilities/ChartImage.axd/{*pathInfo}");
            routes.MapPageRoute("DiagnosticRoute", "Utilities/{page}", "~/Utilities/{page}.aspx");

            routes.MapRoute(
                "Default", // Route name
                "{controller}/{action}/{id}", // URL with parameters
                new { controller = "Home", action = "Index", id = UrlParameter.Optional } // Parameter defaults
            );
        }

        protected void Application_BeginRequest()
        {
            //Remove for production
            EnsureDatabase();
        }

        protected void Application_Start()
        {
            AreaRegistration.RegisterAllAreas();
            RegisterRoutes(RouteTable.Routes);

            // Roles
            CreateRoles();

            // AutoMapper
            AutoMapperConfiguration.Run();

            // StructureMap
            StructureMapBootstrapper.BootstrapStructureMap();

            //Enable Profiler
            //Telerik.OpenAccess.ServiceHost.ServiceHostManager.StartProfilerService("OA.ProfilerPort");
        }

        protected void Application_End()
        {
            //Telerik.OpenAccess.ServiceHost.ServiceHostManager.StopProfilerService();
        }

        // TODO: Temporary hack, until I move somewhere better
        private void CreateRoles()
        {
            // Assign user to specific role
            if (!Roles.RoleExists("Organizer"))
                Roles.CreateRole("Organizer");
            if (!Roles.RoleExists("Speaker"))
                Roles.CreateRole("Speaker");
            if (!Roles.RoleExists("Attendee"))
                Roles.CreateRole("Attendee");


        }
    }
}