﻿using System.Security.Principal;
using EventTracker.DAL;
using EventTracker.Infrastructure;

namespace EventTracker.Helpers
{
    public static class SecurityHelper
    {
        public static bool IsOrganizer(this IPrincipal user)
        {
            return user.IsInRole(RoleConstants.Organizer);
        }

        public static bool IsAttendee(this IPrincipal user)
        {
            return user.IsInRole(RoleConstants.Attendee);
        }

        public static bool IsSpeaker(this IPrincipal user)
        {
            return user.IsInRole(RoleConstants.Speaker);
        }

        public static bool IsOrganizerOfConference(this IPrincipal user, Conference conference)
        {
            if (user.IsOrganizer())
            {
                Person person = Context.PersonRepository.GetByUsername(user.Identity.Name);
                return Context.ConferenceRepository.IsConferenceOrganizer(conference, person);
            }
            else
                return false;
        }

        private static IContextManager Context
        {
            get
            {
                return StructureMapBootstrapper.Container.GetInstance<IContextManager>();
            }
        }

        public static bool CanEditSpeakers(this IPrincipal user)
        {
            return false;
        }

        public static bool CanAddSpeakers(this IPrincipal user)
        {
            return false;
        }

        public static bool CanRemoveSpeakers(this IPrincipal user)
        {
            return false;
        }

        public static bool CanEditConferences(this IPrincipal user)
        {
            return false;
        }

        public static bool CanAddConferences(this IPrincipal user)
        {
            return false;
        }

        public static bool CanRemoveConferences(this IPrincipal user)
        {
            return false;
        }

        public static bool CanEditTracks(this IPrincipal user)
        {
            return false;
        }

        public static bool CanAddTracks(this IPrincipal user)
        {
            return false;
        }

        public static bool CanRemoveTracks(this IPrincipal user)
        {
            return false;
        }

        public static bool CanAddSessions(this IPrincipal user)
        {
            return false;
        }

        public static bool CanRemoveSessions(this IPrincipal user)
        {
            return false;
        }

        public static bool CanEditSessions(this IPrincipal user)
        {
            return false;
        }
    }
}