﻿using System;
using System.Linq;
using Telerik.OpenAccess.Diagnostics;
using System.Drawing;
using Telerik.Web.UI;
using Telerik.Charting;
using Telerik.OpenAccess;
using StructureMap;
using EventTracker.Infrastructure;

namespace EventTracker.Web
{
    public partial class HealthForm : System.Web.UI.Page
    {
        public class CalculatedData
        {
            public int Connection_Obtained { get; set; }
            public int Sql_Query { get; set; }
            public int Sql_Commit { get; set; }
            public int Sql_FetchedRows { get; set; }
            public int Cache_Object_Hits { get; set; }
            public int Cache_Object_Miss { get; set; }
            public int Cache_Query_Hits { get; set; }
            public int Cache_Query_Miss { get; set; }
            public int Sql_Delete { get; set; }
            public int Sql_Insert { get; set; }
            public int Sql_Update { get; set; }
        }

        private ContextManager contextManager;
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!Page.IsPostBack)
            {
                UpdateChartLayout();
            }

            contextManager = ObjectFactory.GetInstance<ContextManager>();
            UpdateData();

        }

        private void UpdateChartLayout()
        {
            // chart displays database read traffic (queries executed, database commit, rollbacks,  rows fetched)
            PrepareChart(RadChartReading, "Database Read Traffic");
            RadChartReading.Series.Add(CreateSerie("Rows Fetched", "Sql_FetchedRows", Color.Black, false));
            RadChartReading.Series.Add(CreateSerie("Con Obtained", "Connection_Obtained", Color.Red));
            RadChartReading.Series.Add(CreateSerie("Queries exec", "Sql_Query", Color.Green));
            RadChartReading.Series.Add(CreateSerie("Commits", "Sql_Commit", Color.Blue));
            AddY2Axis(RadChartReading, "Rows Fetched", Color.Black);

            // chart displays database write traffic ()
            PrepareChart(RadChartWriting, "Database Write Traffic");
            RadChartWriting.Series.Add(CreateSerie("Deletes", "Sql_Delete", Color.Red));
            RadChartWriting.Series.Add(CreateSerie("Inserts", "Sql_Insert", Color.Green));
            RadChartWriting.Series.Add(CreateSerie("Updates", "Sql_Update", Color.Blue));

            PrepareChart(RadChartSecondLevelCache, "Second Level Cache %");
            RadChartSecondLevelCache.Series.Add(CreateSerie("Query Hit %", "L2Cache_Query_HitRatio", Color.Blue));
            RadChartSecondLevelCache.Series.Add(CreateSerie("Query Fill %", "L2Cache_Query_FillRatio", Color.Yellow));
            RadChartSecondLevelCache.Series.Add(CreateSerie("Object Hit %", "L2Cache_Object_HitRatio", Color.Green));
            RadChartSecondLevelCache.Series.Add(CreateSerie("Object Fill %", "L2Cache_Object_FillRatio", Color.Red));

            PrepareChart(RadChartSecondLevelCacheHits, "Second Level Cache Hits");
            RadChartSecondLevelCacheHits.Series.Add(CreateSerie("Query Hits", "Cache_Query_Hits", Color.Blue));
            RadChartSecondLevelCacheHits.Series.Add(CreateSerie("Query Misses", "Cache_Query_Miss", Color.Yellow));
            RadChartSecondLevelCacheHits.Series.Add(CreateSerie("Object Hits", "Cache_Object_Hits", Color.Green));
            RadChartSecondLevelCacheHits.Series.Add(CreateSerie("Object Misses", "Cache_Object_Miss", Color.Red));

            PrepareChart(RadChartConnectionPool, "Connection Pool");
            RadChartConnectionPool.Series.Add(CreateSerie("Blocked", "Connection_ObtainedWait", Color.Black, false));
            RadChartConnectionPool.Series.Add(CreateSerie("Active", "Connection_Active", Color.Red));
            RadChartConnectionPool.Series.Add(CreateSerie("Max Active", "Connection_MaxActive", Color.Blue));
            RadChartConnectionPool.Series.Add(CreateSerie("Idle", "Connection_Idle", Color.Green));
            RadChartConnectionPool.Series.Add(CreateSerie("Validated", "Connection_Validate", Color.Yellow, false));
            AddY2Axis(RadChartConnectionPool, "Blocked\r\nValidated", Color.Black);

            PrepareChart(RadChartScopePool, "Context Pool");
            RadChartScopePool.Series.Add(CreateSerie("Active", "Context_Active", Color.Red));
            RadChartScopePool.Series.Add(CreateSerie("Max Active", "Context_MaxActive", Color.Blue));
            RadChartScopePool.Series.Add(CreateSerie("Idle", "Context_Idle", Color.Green));
        }

        private void UpdateData()
        {
            var l = Get(120);
            if (l != null && l.Count > 2)
            {
                var list = l.ToList();
                DateTime start = l[0].Timestamp;
                DateTime middle = l[l.Count / 2].Timestamp;
                DateTime end = l[l.Count - 1].Timestamp;
                var transfo = new DatabaseMetricsTransformation<CalculatedData>(l);
                transfo.Calculate("Connection_Obtained", "MetricDelta", "Unpooled Connections");
                transfo.Calculate("Sql_Query", "MetricDelta", "Queries");
                transfo.Calculate("Sql_Commit", "MetricDelta", "Commits");
                transfo.Calculate("Sql_FetchedRows", "MetricDelta", "Fetched Rows");
                transfo.Calculate("Cache_Object_Hits", "MetricDelta", "State Cache Hits");
                transfo.Calculate("Cache_Object_Miss", "MetricDelta", "State Cache Misses");
                transfo.Calculate("Cache_Query_Hits", "MetricDelta", "Query Cache Hits");
                transfo.Calculate("Cache_Query_Miss", "MetricDelta", "Query Cache Misses");
                transfo.Calculate("Sql_Delete", "MetricDelta", "Deletes");
                transfo.Calculate("Sql_Insert", "MetricDelta", "Inserts");
                transfo.Calculate("Sql_Update", "MetricDelta", "Updates");


                AddXAxisLabels(RadChartReading, list.Count, start, middle, end);
                RadChartReading.DataSource = transfo;
                RadChartReading.DataBind();

                AddXAxisLabels(RadChartWriting, list.Count, start, middle, end);
                RadChartWriting.DataSource = transfo;
                RadChartWriting.DataBind();

                AddXAxisLabels(RadChartSecondLevelCache, list.Count, start, middle, end);
                RadChartSecondLevelCache.DataSource = list;
                RadChartSecondLevelCache.DataBind();

                AddXAxisLabels(RadChartSecondLevelCacheHits, list.Count, start, middle, end);
                RadChartSecondLevelCacheHits.DataSource = transfo;
                RadChartSecondLevelCacheHits.DataBind();

                AddXAxisLabels(RadChartConnectionPool, list.Count, start, middle, end);
                RadChartConnectionPool.DataSource = list;
                RadChartConnectionPool.DataBind();

                AddXAxisLabels(RadChartScopePool, list.Count, start, middle, end);
                RadChartScopePool.DataSource = list;
                RadChartScopePool.DataBind();

            }
        }

        private void AddY2Axis(RadChart chart, string title, Color color)
        {
            chart.PlotArea.YAxis2.AxisLabel.TextBlock.Text = title;
            chart.PlotArea.YAxis2.AxisLabel.TextBlock.Appearance.TextProperties.Color = color;
            chart.PlotArea.YAxis2.AxisLabel.Visible = true;
            chart.PlotArea.YAxis2.AxisLabel.Appearance.Position.AlignedPosition = Telerik.Charting.Styles.AlignedPositions.TopLeft;
            chart.PlotArea.YAxis2.VisibleValues = Telerik.Charting.Styles.ChartAxisVisibleValues.Positive;
            chart.PlotArea.YAxis2.Appearance.MajorGridLines.Visible = false;
            chart.PlotArea.YAxis2.Appearance.MinorGridLines.Visible = false;
        }

        private void PrepareChart(RadChart chart, string title)
        {
            chart.Legend.Appearance.Position.AlignedPosition = Telerik.Charting.Styles.AlignedPositions.BottomRight;
            chart.Width = new System.Web.UI.WebControls.Unit(700);
            chart.Height = new System.Web.UI.WebControls.Unit(250);
            chart.ChartTitle.TextBlock.Text = title;
            chart.PlotArea.Appearance.Dimensions.Margins.Left = new Telerik.Charting.Styles.Unit(5, Telerik.Charting.Styles.UnitType.Percentage);
            chart.PlotArea.Appearance.Dimensions.Margins.Right = new Telerik.Charting.Styles.Unit(22, Telerik.Charting.Styles.UnitType.Percentage);
            chart.PlotArea.Appearance.Dimensions.Margins.Bottom = new Telerik.Charting.Styles.Unit(10, Telerik.Charting.Styles.UnitType.Percentage);
            chart.PlotArea.Appearance.Dimensions.Margins.Top = new Telerik.Charting.Styles.Unit(5, Telerik.Charting.Styles.UnitType.Percentage);
            //chart.ChartTitle.Appearance.Position.AlignedPosition = Telerik.Charting.Styles.AlignedPositions.Left;
            //chart.ChartTitle.Appearance.RotationAngle = -90;
            chart.EnableViewState = false;
            //chart.AutoLayout = true;
            chart.Series.Clear();
            chart.PlotArea.YAxis.VisibleValues = Telerik.Charting.Styles.ChartAxisVisibleValues.Positive;
            chart.PlotArea.YAxis.Appearance.MinorGridLines.Visible = false;
            chart.PlotArea.XAxis.Appearance.MinorGridLines.Visible = false;
            chart.PlotArea.XAxis.Appearance.MajorGridLines.Visible = false;
            //chart.ClientSettings.ScrollMode = ChartClientScrollMode.XOnly;
        }

        private void AddXAxisLabels(RadChart chart, int count, DateTime start, DateTime middle, DateTime end)
        {
            chart.PlotArea.XAxis.AutoScale = false;
            chart.PlotArea.XAxis.Clear();
            chart.PlotArea.XAxis.LayoutMode = Telerik.Charting.Styles.ChartAxisLayoutMode.Normal;
            for (int i = 0; i < count; i++)
            {
                chart.PlotArea.XAxis.AddItem(new ChartAxisItem());
                chart.PlotArea.XAxis[i].TextBlock.Text = " ";
            }
            chart.PlotArea.XAxis[0].TextBlock.Text = start.ToString("HH:mm:ss");
            chart.PlotArea.XAxis[(count / 2)].TextBlock.Text = middle.ToString("HH:mm:ss");
            chart.PlotArea.XAxis[count - 1].TextBlock.Text = end.ToString("HH:mm:ss");
        }

        private Telerik.Charting.ChartSeries CreateSerie(string legendText, string propertyName, Color color, bool primary)
        {
            var ser = CreateSerie(legendText, propertyName, color);
            if (primary == false)
                ser.YAxisType = Telerik.Charting.ChartYAxisType.Secondary;
            return ser;
        }

        private Telerik.Charting.ChartSeries CreateSerie(string legendText, string propertyName, Color color)
        {
            var ser = CreateSerie(legendText, propertyName);
            ser.Appearance.LineSeriesAppearance.Color = color;
            return ser;
        }

        private Telerik.Charting.ChartSeries CreateSerie(string legendText, string propertyName)
        {
            var ser = new Telerik.Charting.ChartSeries(legendText);
            ser.DataYColumn = propertyName;
            ser.Appearance.LabelAppearance.Visible = false;
            ser.Type = Telerik.Charting.ChartSeriesType.Line;
            return ser;
        }

        private DatabaseMetricsCollection Get(int count)
        {

            return new History(this.contextManager.Context).GetDatabaseMetrics(null, null, count * -1);
        }

        
    }
}